.setcpu "65816"
.smart

; See LoadGraphics.asm
.import LoadVRAM, DMAPalette

;
; LoadPalette: loads palette information into CGRAM
;
; arg0: src   -- address of source data
; arg1: start -- color to start at (8-bit value)
; arg2: size: -- number of colors to copy (8 or 16-bit value)
;
; Modifies: A, X
; Requires: A=8, X/Y=16
;
.macro LoadPalette src, start, size
.if .PARAMCOUNT <> 3
.error "LoadPalette: incorrect number of parameters"
.endif
  lda #start                 ; Color # to start at
  sta $2121
  lda #.BANKBYTE(src)        ; Bank of src
  ldx #.LOWORD(src)          ; Address of src
  ldy #(size*2)              ; Every color is 2 bytes
  jsr DMAPalette             ; see main.asm
.endmacro


;
; LoadBlockToVRAM: simplifies calling LoadVRAM to copy data to VRAM
;
; arg0: src    -- address of source data
; arg1: dest   -- VRAM address to write to (16-bit address)
; arg2: size   -- number of bytes to copy (8 or 16-bit value)
; arg3: offset -- [optional] offset to add to src (16-bit value)
;
; NOTE: src+offset could wrap from $FFFF->$0000 within the same bank.  This
; is acceptable; SNES DMA cannot handle bank wrapping anyway.
;
; Modifies: A, X, Y
; Requires: A=8, X/Y=16
;
.macro LoadBlockToVRAM src, dst, size, offset
.if .PARAMCOUNT < 3 .or .PARAMCOUNT > 4
.error "LoadBlockToVRAM: incorrect number of parameters"
.endif
  lda #%10000000             ; Increment VRAM address by 1 after write to $2119
  sta $2115
  ldx #.LOWORD(dst)
  stx $2116                  ; $2116: Word address for accessing VRAM
  lda #.BANKBYTE(src)        ; Bank of src
.if .PARAMCOUNT = 4
  ldx #.LOWORD(src)+offset   ; Address of src + offset
.else
  ldx #.LOWORD(src)          ; Address of src
.endif
  ldy #size                  ; Length of data to copy
  jsr LoadVRAM               ; see main.asm
.endmacro


;----------------------------------------------------------------------------
; ClearVRAM -- Sets every byte of VRAM to zero
; In: None
; Out: None
; Modifies: A/X
;----------------------------------------------------------------------------
.macro ClearVRAM
   LDA #$80
   STA $2115         ;Set VRAM port to word access
   LDX #$1809
   STX $4300         ;Set DMA mode to fixed source, WORD to $2118/9
   LDX #$0000
   STX $2116         ;Set VRAM port address to $0000
   LDX #.LOWORD(zero_fill_byte)
   STX $4302         ;Set source address to $xx:0000
   LDA #.BANKBYTE(zero_fill_byte)
   STA $4304         ;Set source bank
   LDX #$0000
   STX $4305         ;Set transfer size to 65536 bytes
   LDA #$01
   STA $420B         ;Initiate transfer
.endmacro


;----------------------------------------------------------------------------
; ClearPalette -- Reset all palette colors to zero
; In: None
; Out: None
; Modifies: X
;----------------------------------------------------------------------------
.macro ClearPalette
   STZ $2121
   LDX #$0100
ClearPaletteLoop:
   STZ $2122
   STZ $2122
   DEX
   BNE ClearPaletteLoop
.endmacro


; Original source: InitSNES2.asm

; InitSNES macro removed -- ridiculous given how it was designed (especially
; the JSL $008000 part -- that's bound to fail at some point, especially
; when moving from MODE 20 to MODE 21 or some other memory model).  Instead,
; just rely on the InitializeSNES routine.

; Reason this is a macro rather than a subroutine: this routine happens to
; zero ALL of the SNES RAM on the system.  And that's good!  But that also
; means we trash the stack...
;
; So rather than futz about with "backing up" the return addresses using
; special memory locations, just use a macro, thus no need for JSL/JSR.
;
.macro InitializeSNES
  sei          ; Disable interrupts
  clc          ; Clear carry, used by XCE
  xce          ; Set 65816 native mode
  jmp :+       ; Needed to set K (bank of PC) properly if MODE 21 is ever used;
               ; see official SNES developers docs, "Programming Cautions"

: cld          ; Disable decimal mode
  phk          ; Push K (PC bank)
  plb          ; Pull B (data bank, i.e. data bank now equals PC bank)

  rep #$30     ; A=16, X/Y=16

  ; Note: this should correlate with ZEROPAGE in lorom256k.cfg
  lda #$0000
  tcd          ; Set D = $0000 (direct page)

  ; Note: this should correlate with the top of BSS in lorom256k.cfg
  ldx #$1fff
  txs          ; Set X = $1fff (stack pointer)

  ;
  ; Register initialisation values, per official Nintendo documentation
  ; Note: if you are looking at this code and thinking "how can I optimise"
  ; this?", then you are doing nothing useful with your time.  This code is
  ; called ONCE on reset or power-on, it does not need to be optimised.  I
  ; (koitsu) find it remarkable how many people have tried to "optimise"
  ; this routine.  STOP IT.
  ;
  sep #$20     ; A=8
  lda #$80
  sta $2100
  stz $2101
  stz $2102
  stz $2103
  stz $2104
  stz $2105
  stz $2106
  stz $2107
  stz $2108
  stz $2109
  stz $210a
  stz $210b
  stz $210c
  stz $210d
  stz $210d
  stz $210e
  stz $210e
  stz $210f
  stz $210f
  stz $2110
  stz $2110
  stz $2111
  stz $2111
  stz $2112
  stz $2112
  stz $2113               
  stz $2113               
  stz $2114
  stz $2114
  lda #$80
  sta $2115
  stz $2116
  stz $2117
  stz $211a
  stz $211b
  lda #$01
  sta $211b
  stz $211c
  stz $211c
  stz $211d
  stz $211d
  stz $211e       
  lda #$01
  sta $211e
  stz $211f
  stz $211f
  stz $2120
  stz $2120
  stz $2121
  stz $2123
  stz $2124
  stz $2125
  stz $2126
  stz $2127
  stz $2128
  stz $2129
  stz $212a
  stz $212b
  stz $212c
  stz $212d
  stz $212e
  stz $212f
  stz $4200
  lda #$ff
  sta $4201
  stz $4202
  stz $4203
  stz $4204
  stz $4205
  stz $4206
  stz $4207
  stz $4208
  stz $4209
  stz $420a
  stz $420b
  stz $420c
  stz $420d

  ClearVRAM          ; Zero VRAM.    Requires A=8, X/Y=16
  ClearPalette       ; Zero palette. Requires A=8, X/Y=16
  
  ;**** clear Sprite tables ********

  STZ $2102	;sprites initialized to be off the screen, palette 0, character 0
  STZ $2103
  LDX #$0080
  LDA #$F0
_Loop08:
  STA $2104	;set X = 240
  STA $2104	;set Y = 240
  STZ $2104	;set character = $00
  STZ $2104	;set priority=0, no flips
  DEX
  BNE _Loop08

  LDX #$0020
_Loop09:
  STZ $2104		;set size bit=0, x MSB = 0
  DEX
  BNE _Loop09

  ;**** clear SNES RAM ********

  STZ $2181		;set WRAM address to $000000
  STZ $2182
  STZ $2183

  LDX #$8008
  STX $4300         ;Set DMA mode to fixed source, BYTE to $2180
  LDX #.LOWORD(zero_fill_byte)
  STX $4302         ;Set source offset
  LDA #.BANKBYTE(zero_fill_byte)
  STA $4304         ;Set source bank
  LDX #$0000
  STX $4305         ;Set transfer size to 64KBytes (65536 bytes)
  LDA #$01
  STA $420B         ;Initiate transfer

  LDA #$01          ;now zero the next 64KB (i.e. 128KB total)
  STA $420B         ;Initiate transfer
.endmacro


;
; Byte used to zero/initialise all of memory in above macro
; This is needed because DMA requires an address of a byte to
; read from, not a raw value.
;
.segment "RODATA"
zero_fill_byte:
  .byte $00
